/*                                                                           a
 *   This file is part of the ESO X-shooter Pipeline                         *
 *   Copyright (C) 2006 European Southern Observatory                        *
 *                                                                           *
 *   This library is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the Free Software             *
 *   Foundation, 51 Franklin St, Fifth Floor, Boston, MA  02111-1307  USA    *
 *                                                                           */

/*
 * $Author: mwhite, nfernand $
 * $Date: 2023-11-07 $
 * $Revision: 0.1 $
 * $Name: not supported by cvs2svn $
 */

#include <cpl.h>
#include <hdrl_spectrum.h>
#include <hdrl_image.h>


typedef struct {
    cpl_array* wavl_air ;
    cpl_array* telluric ;
    cpl_array* flux ;
} molecfit_ref ;

typedef struct {
    double x_precision ;
    double offset_from_optimal ;
    double what_we_have ;
    double best_we_can ;
    double amplitude_given_position ;
    double lsf_off;
    double lsf_rms;
} line_analysis ;

struct fitting_data {
    size_t n ;
    hdrl_spectrum1D* order_spec ;
    hdrl_spectrum1D* tell_spec ;
};

#define XSH_WAVL_CORR_GLOBAL_WD 0.5
#define XSH_WAVL_CORR_SENS_LEVEL 0.1

#define XSH_WAVL_CORR_MAXDEG_1 5
#define XSH_WAVL_CORR_MAXDEG_2 0
#define XSH_WAVL_CORR_MAXDEG_SWITCH 7 // First order using MAXDEG_2

#define XSH_WAVL_NM_PIX 0.06  // nm->pixel scale conversion

int xsh_wc_get_maxdeg(int order);

hdrl_spectrum1D* xsh_wc_molecfit_ref_to_hdrl_spectrum1D(molecfit_ref* mf);

molecfit_ref* xsh_wc_load_refdata(char* filename) ;

int xsh_wc_wavl_vac_to_air(cpl_array* wavls) ;

cpl_array* xsh_wc_find_tell_lines(molecfit_ref* refdata) ;

hdrl_spectrum1D* xsh_wc_get_data_order(cpl_frame* order_tab, int order) ;

line_analysis* xsh_wc_analyze_line(
        molecfit_ref* telluric_data,
        hdrl_spectrum1D* order_dat,
        int order,
        double line_loc
        );

cpl_error_code xsh_wc_normalize_flux_array(hdrl_image* flux);

cpl_error_code xsh_wc_structure_output_table(cpl_table* tab);

cpl_error_code xsh_wc_extract_polynomial(cpl_table* tab, int order,
                                          cpl_polynomial** poly);

cpl_array* xsh_wc_shift_wavls_poly(hdrl_spectrum1D* orig_spec,
                                   cpl_polynomial* poly);

cpl_error_code xsh_wc_update_res1d_frame(cpl_frame* res1D_frame,
                                         cpl_frame* res1Deso_frame,
                                         hdrl_spectrum1D* order_specs[]);

cpl_error_code xsh_wc_analyze_telluric(cpl_frame* mf_reference_frame,
                                       cpl_frame* res1D_frame,
                                       cpl_frame** ord_wl_offsets_tab);

cpl_error_code xsh_wc_correct_for_telluric(cpl_frame* ord_wl_offsets_tab,
                                           cpl_frame** res1D_frame,
                                           cpl_frame** res1Deso_frame);

